import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
#
#        Exc el形式データファイルの読み込み
#
infl_nm = input('Input data file (*xlsx) = ')
with pd.ExcelFile(infl_nm) as f_xlsx:   
    data_xlsx = pd.read_excel(f_xlsx, header=None)                              #   書き出し

#   データ値をndarray型の配列に変換
data = data_xlsx.values
print('\n', data)

W = data[1:,1]
Y = data[1:,2]
print('W =\n',W)
print('Y =\n',Y)

Yt = Y[W==1]                #    Treated group
Yc = Y[W==0]                #    Control group
print('Yt =\n', Yt)
print('Yc =\n', Yc)

class CumDist:
    """
            Cumulative distribution
    """
    def __init__(self, y):
        self.N = len(y)
        self.y = sorted(y)

    def cdf(self, v):
        pos = 0
        while True:
            if v < self.y[pos]:
                break
            if pos < self.N:
                pos += 1
            if pos >= self.N:
                break
        return pos / self.N


min_y = np.min(Y)
max_y = np.max(Y)
cum_yt = CumDist(Yt)      #   Cumulative distribution for Yt
x_coord = np.arange(min_y-1, max_y+1, 0.01)
yt_coord = [cum_yt.cdf(v) for v in x_coord]
plt.plot(x_coord, yt_coord, label='treated')

cum_yc = CumDist(Yc)      #   Cumulative distributin for Yc
yc_coord = [cum_yc.cdf(v) for v in x_coord]
plt.plot(x_coord, yc_coord, label='control')
plt.legend()
plt.show()

CumVs_t = np.array([cum_yt.cdf(v) for v in Y])
CumVs_c = np.array([cum_yc.cdf(v) for v in Y])

Tks = np.max(np.abs(CumVs_t - CumVs_c))

npg_shuffle = np.random.default_rng().shuffle     #   Shuffling用関数
K = 1_000_000
cnt = 1
for i in range(K-1):
    npg_shuffle(W)
    Yt = Y[W==1]
    Yc = Y[W==0]

    cum_yt = CumDist(Yt)  
    cum_yc = CumDist(Yc)

    CumVs_t = np.array([cum_yt.cdf(v) for v in Y])
    CumVs_c = np.array([cum_yc.cdf(v) for v in Y])
    diff_max = np.max(np.abs(CumVs_t - CumVs_c))
    if diff_max >= Tks:
        cnt += 1
    if i % 10000 == 0 and i > 0:
        print(f'{(i+1):,}/{K:,}     {cnt/(i+1):.5f},    {diff_max:5f}')

p_value = cnt / K
s_e = (p_value * (1.0 - p_value) / K) ** 0.5
print(f'\nTks = {Tks:.3f}    p-value = {p_value:.4f}   s.e. = {s_e:.5f}')

